@@ -43,6 +43,11 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifdef G_OS_WIN32
+#include <io.h>
+#include <windows.h>
+#endif
+
 /****************** ORBit-specific stuff ****************/
 
 #include <orbit/orbit.h>
@@ -366,11 +371,35 @@ static const GOptionEntry bonobo_activation_goption_options[] = {
         { NULL }
 };
 
+static void
+init_gettext (gboolean bind_codeset)
+{
+	static gboolean initialized = FALSE;
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET	
+	static gboolean codeset_bound = FALSE;
+#endif
+
+	if (!initialized)
+	{
+		bindtextdomain (GETTEXT_PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
+		initialized = TRUE;
+	}
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET	
+	if (!codeset_bound && bind_codeset)
+	{
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+		codeset_bound = TRUE;
+	}
+#endif
+}
+
 GOptionGroup *
 bonobo_activation_get_goption_group (void)
 {
 	GOptionGroup *group;
 
+	init_gettext (TRUE);
+
 	group = g_option_group_new ("bonobo-activation",
 				    N_("Bonobo Activation"),
 				    N_("Show Bonobo Activation options"),
@@ -405,7 +434,7 @@ bonobo_activation_ior_fd_get (void)
 void
 bonobo_activation_preinit (gpointer app, gpointer mod_info)
 {
-        bindtextdomain (GETTEXT_PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
+	init_gettext (FALSE);
 }
 
 void
@@ -417,6 +446,19 @@ bonobo_activation_postinit (gpointer app, gpointer mod_info)
 	if (bonobo_activation_ior_fd > 2)
 		fcntl (bonobo_activation_ior_fd, F_SETFD, FD_CLOEXEC);
 #endif
+#ifdef G_OS_WIN32
+	if (bonobo_activation_ior_fd > 2) {
+                HANDLE newhandle;
+
+                if (DuplicateHandle (GetCurrentProcess (), (HANDLE) _get_osfhandle (bonobo_activation_ior_fd),
+                                     GetCurrentProcess (), &newhandle,
+                                     0, FALSE, DUPLICATE_SAME_ACCESS)) {
+                        close (bonobo_activation_ior_fd);
+                        bonobo_activation_ior_fd = _open_osfhandle ((int) newhandle, _O_TEXT | _O_NOINHERIT);
+                }
+        }
+#endif
+
         if (bonobo_activation_activate_iid)
                 g_timeout_add_full (G_PRIORITY_LOW,
                                     BONOBO_ACTIVATION_FACTORY_TIMEOUT,
@@ -467,8 +509,8 @@ bonobo_activation_is_initialized (void)
 char *
 bonobo_activation_get_popt_table_name (void)
 {
-        bindtextdomain (PACKAGE, BONOBO_ACTIVATION_LOCALEDIR);
-        return _("Bonobo activation options");
+	init_gettext (FALSE);
+	return _("Bonobo activation options");
 }
 
 
